/*
    MapShow2: simple visualisation of pre-downsampled Terra MODIS datasets
    Copyright (C) 2002,2004 John Sullivan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "mainfrm.h"

#include <stdio.h>
#include <math.h>

void MyTrace(LPCTSTR fmt, ...)
{
    va_list args;
    va_start (args, fmt);

    TCHAR buf[512];
    _vsntprintf(buf, 512, fmt, args);
    OutputDebugString(buf);

    va_end(args);
}

//#define MYTRACE MyTrace
#define MYTRACE 1 ? (void)0 : MyTrace
//#define MYTRACE2 MyTrace
#define MYTRACE2 1 ? (void)0 : MyTrace

extern LPTSTR cmdLine;

struct map_def map_defs[] = {
    { _T("MOD09A1.W.interpol.cyl.retouched.3x1350x1350"), _T("MOD09A1.E.interpol.cyl.retouched.3x1350x1350"), 1350, 1350, 3 },
    { _T("landcover.W.675x675"), _T("landcover.E.675x675"), 675, 675, 1 }
};

const long SCRL_FAC = 70;
const long SCRL_TIM = 15;
const double M_PI = 3.1415926535;
const int L_LEN = 2048;


COLORREF mapcols[16] = {
#if 0
    RGB(0, 0, 128),     // 0  water
    RGB(0, 224, 0),     // 1  broadleaf evergreen forest
    RGB(0, 128, 0),     // 2  coniferous evergreen forest and woodland
    RGB(0, 128, 64),    // 3  high latitude deciduous forest and woodland
    RGB(224, 224, 255), // 4  tundra
    RGB(64, 224, 64),   // 5  mixed coniferous forest and woodland
    RGB(128, 224, 0),   // 6  wooded grassland
    RGB(64, 224, 0),    // 7  grassland
    RGB(128, 64, 0),    // 8  bare ground
    RGB(128, 96, 0),    // 9  shrubs and bare ground
    RGB(224, 224, 0),   // 10 cultivated crops
    RGB(0, 255, 0),     // 11 broadleaf deciduous forest and woodland
    RGB(255, 0, 0),     // 12 ?
    RGB(255, 0, 255),   // 13 ?
#endif
    RGB(24, 24, 128),     // 0  Water
    RGB(0, 32, 16),       // 1  Evergreen Needleleaf Forests
    RGB(0, 64, 64),       // 2  Evergreen Broadleaf Forests
    RGB(0, 128, 0),       // 3  Deciduous Needleleaf Forests
    RGB(128, 192, 16),    // 4  Deciduous Broadleaf Forests
    RGB(64, 190, 64),     // 5  Mixed Forests
    RGB(96, 64, 0),       // 6  Woodlands
    RGB(214, 148, 21),    // 7  Wooded Grasslands/Shrubs
    RGB(255, 221, 51),    // 8  Closed Bushlands or Shrublands
    RGB(176, 209, 41),    // 9  Open Shrublands
    RGB(157, 211, 112),   // 10 Grasses
    RGB(190, 96, 56),     // 11 Croplands
//    RGB(102, 102, 102), // 12 Bare
//    RGB(70, 78, 140),   // 13 Mosses and Lichens
    RGB(192, 192, 64),    // 12 Bare
    RGB(102, 102, 102),   // 13 Urban and Built-up
};

CMainFrame::CMainFrame()
{
    m_bDDStarted = false;   
    m_bTimerOn = false;
    m_bInUser = false;
    m_bInDrag = false;

    m_nmaps = sizeof(map_defs)/sizeof(struct map_def);
    m_buf = (BYTE**)malloc(sizeof(BYTE*)*m_nmaps);
    memset(m_buf, 0, sizeof(BYTE*)*m_nmaps);

    m_alpha = (int**)malloc(sizeof(int*)*m_nmaps);
    memset(m_alpha, 0, sizeof(int*)*m_nmaps);

    m_curmap = -1;

    for ( int def=0 ; def<m_nmaps ; def++ ) {
        HANDLE hin1 = CreateFile(map_defs[def].westpath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
        if (hin1==INVALID_HANDLE_VALUE) continue;
        HANDLE hin2 = CreateFile(map_defs[def].eastpath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
        if (hin2==INVALID_HANDLE_VALUE) {
            CloseHandle(hin1);
            continue;
        }

        m_buf[def] = (BYTE*)malloc(2*map_defs[def].xdim*map_defs[def].ydim*map_defs[def].bpp);
        memset(m_buf[def], 0, 2*map_defs[def].xdim*map_defs[def].ydim*map_defs[def].bpp);

        int x, y;

        for ( y=0 ; y<map_defs[def].ydim ; y++ ) {
            DWORD dwRead=0;
            ReadFile(hin1, m_buf[def]+y*2*map_defs[def].xdim*map_defs[def].bpp, map_defs[def].xdim*map_defs[def].bpp, &dwRead, NULL);
            ReadFile(hin2, m_buf[def]+(y*2+1)*map_defs[def].xdim*map_defs[def].bpp, map_defs[def].xdim*map_defs[def].bpp, &dwRead, NULL);
        }

        CloseHandle(hin1);
        CloseHandle(hin2);

        m_alpha[def] = (int*)malloc(sizeof(int)*L_LEN);
        memset(m_alpha[def], 0, sizeof(int)*L_LEN);

        for ( x=0 ; x<L_LEN ; x++ ) {
            double alpha = acos(1.0-(double)(2*x)/(double)L_LEN);
            m_alpha[def][x] = (int)((double)(map_defs[def].xdim-1)*alpha/M_PI);
        }

        if (m_curmap==-1) m_curmap = def;
    }

    m_loff = 0;
}

CMainFrame::~CMainFrame()
{
    if (m_hWnd) DestroyWindow();

    if (m_buf) {
        for ( int def=0 ; def<m_nmaps ; def++ ) {
            if (m_buf[def]) free(m_buf[def]);
        }
        free(m_buf);
    }
    m_buf = NULL;

    if (m_alpha) {
        for ( int def=0 ; def<m_nmaps; def++ ) {
            if (m_alpha[def]) free(m_alpha[def]);
        }
        free(m_alpha);
    }
    m_alpha = NULL;
}

LRESULT CMainFrame::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
//		CreateSimpleStatusBar();
//		UISetCheck(ID_VIEW_STATUS_BAR, 1);

	// register object for message filtering and idle updates
	CMessageLoop* pLoop = _Module.GetMessageLoop();
	ATLASSERT(pLoop != NULL);
	pLoop->AddMessageFilter(this);
	pLoop->AddIdleHandler(this);

    SetTimer(100, SCRL_TIM);
    m_bTimerOn = true;

	return 0;
}

void CMainFrame::InitDD()
{
    if (m_spS1!=NULL) m_spS1.Release();
    if (m_spS2!=NULL) m_spS2.Release();
    if (m_spC!=NULL) m_spC.Release();
    if (m_spP!=NULL) m_spP.Release();
    if (m_spDD!=NULL) m_spDD.Release();

    HRESULT hr = DirectDrawCreateEx(NULL, (void**)&m_spDD, IID_IDirectDraw7, NULL);

    hr = m_spDD->SetCooperativeLevel(NULL, DDSCL_NORMAL);

    int i;

    PALETTEENTRY ddpal[16];
    for ( i=0 ; i<16 ; i++ ) *(COLORREF*)(ddpal+i) = mapcols[i];
    hr = m_spDD->CreatePalette(DDPCAPS_4BIT, ddpal, &m_spP, NULL);

    RECT rc;
    GetClientRect(&rc);
    
    DDSURFACEDESC2 ddsd;
    memset(&ddsd, 0, sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
    hr = m_spDD->CreateSurface(&ddsd, &m_spS1, NULL);

    ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_3DDEVICE;
    ddsd.dwWidth = rc.right;
    ddsd.dwHeight = rc.bottom;
    hr = m_spDD->CreateSurface(&ddsd, &m_spS2, NULL);

    hr = m_spDD->CreateClipper(0, &m_spC, NULL);
    m_spC->SetHWnd(0, m_hWnd);
    m_spS1->SetClipper(m_spC);
    m_spC.Release();
    
    m_spS1->SetPalette(m_spP);
}

LRESULT CMainFrame::OnEraseBkgnd(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    return TRUE;
}

LRESULT CMainFrame::OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    if (m_bTimerOn && !m_bInUser) {
        long res = ((long)m_loff-(long)map_defs[m_curmap].xdim/(long)SCRL_FAC) % (long)(2*map_defs[m_curmap].xdim);
        if (res<0) res = 2*map_defs[m_curmap].xdim+res;
        m_loff = res;

        Invalidate();
    }

    return 0;
}

LRESULT CMainFrame::OnMouseWheel(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    double delta = (3.0*(double)(short)HIWORD(wParam))/120.0;

    double offset = (delta*(double)map_defs[m_curmap].xdim)/(double)SCRL_FAC;

    long res = ((long)m_loff-(long)offset) % (long)(2*map_defs[m_curmap].xdim);
    if (res<0) res = 2*map_defs[m_curmap].xdim+res;

    m_loff = res;

    Invalidate();

    return 0;
}

LRESULT CMainFrame::OnMiddleClick(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    m_bTimerOn = !m_bTimerOn;

    return 0;
}

LRESULT CMainFrame::OnKeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    switch (wParam) {
    case VK_LEFT:
    case VK_DOWN:
        return OnMouseWheel(WM_MOUSEWHEEL, MAKELONG(0, -120), 0, bHandled);
    case VK_RIGHT:
    case VK_UP:
        return OnMouseWheel(WM_MOUSEWHEEL, MAKELONG(0, 120), 0, bHandled);
    case VK_SPACE:
        m_bTimerOn = !m_bTimerOn;
        return 0;
    }

    bHandled = FALSE;
    return 0;
}

LRESULT CMainFrame::OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    POINT pt = { GET_X_LPARAM(lParam)-m_x0, GET_Y_LPARAM(lParam)-m_y0 };

    if (pt.y>=-m_ry && pt.y<=m_ry) {

        long rxy = (long)sqrt((double)(m_rx*m_rx)*(1.0-((double)(pt.y*pt.y)/(double)(m_ry*m_ry))));

        if (pt.x>=-rxy && pt.x<=rxy) {
            SetCapture();
            m_bInUser = true;
            m_bInDrag = true;

            m_ooff = m_loff;
            m_doff = m_alpha[m_curmap][(L_LEN-1)*(pt.x+rxy)/(rxy*2)];
//            m_doff = (m_doff+m_loff) % (2*in_X);
        }
    }

    return 0;
}

LRESULT CMainFrame::OnLButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    if (m_bInDrag) {
        ReleaseCapture();
        m_bInDrag = false;
        m_bInUser = false;
    }

    return 0;
}

LRESULT CMainFrame::OnMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    if (m_bInDrag) {
        POINT pt = { GET_X_LPARAM(lParam)-m_x0, GET_Y_LPARAM(lParam)-m_y0 };

        long off = m_doff;

        if (pt.y>=-m_ry && pt.y<=m_ry) {
            long rxy = (long)sqrt((double)(m_rx*m_rx)*(1.0-((double)(pt.y*pt.y)/(double)(m_ry*m_ry))));

            if (pt.x>=-rxy && pt.x<=rxy) {
                off = m_alpha[m_curmap][(L_LEN-1)*(pt.x+rxy)/(rxy*2)];
            } else {
                if (pt.x<-rxy) off = 0;
                if (pt.x>rxy) off = map_defs[m_curmap].xdim;
            }
        } else {
            if (pt.x<0) off = 0;
            if (pt.x>0) off = map_defs[m_curmap].xdim;
        }

        if (off!=m_doff) {
            m_loff = (m_ooff+m_doff-off) % (2*map_defs[m_curmap].xdim);
            if (m_loff<0) m_loff = 2*map_defs[m_curmap].xdim+m_loff;

            Invalidate();
        }
    }

    return 0;
}

void CMainFrame::DDHLine(DDSURFACEDESC2 *pddsd, int yt, int yb, int y, int x0, int r)
{
    BYTE *ps = (BYTE*)pddsd->lpSurface;
    BYTE *psl = ps + pddsd->lPitch*y;

    DWORD pixsz = (pddsd->ddpfPixelFormat.dwRGBBitCount+7)/8;

//    int by = (in_Y-1)*(y-yt)/(yb-yt);
    long by = m_alpha[m_curmap][(L_LEN-1)*(y-yt)/(yb-yt)];
    BYTE *pb = m_buf[m_curmap]+(map_defs[m_curmap].xdim*2*map_defs[m_curmap].bpp*by);

    BYTE *pslp = psl + (x0-r)*pixsz;

    for ( int x=x0-r ; x<=x0+r ; x++ ) {
//        double alpha = acos(((double)x0-(double)x)/(double)r);
//        const double M_PI = 3.1415926535;
//        int bx = (int)((double)(in_X-1)*alpha/M_PI);

        long bx = m_alpha[m_curmap][(L_LEN-1)*(x-x0+r)/(r*2)];

        int ci = 0;
        COLORREF c = 0;

//        int bx = (in_X-1)*(x-(x0-r))/(2*r);

        if (map_defs[m_curmap].bpp==1) {
            ci = pb[(bx+m_loff) % (2*map_defs[m_curmap].xdim)];
            c = mapcols[ci];
            switch (pddsd->ddpfPixelFormat.dwRGBBitCount) {
            case 8:
                *pslp++ = ci;
                break;

            case 24:
                *pslp++ = (BYTE)((c & 0xff0000)>>16);
                *pslp++ = (BYTE)((c & 0xff00)>>8);
                *pslp++ = (BYTE)(c & 0xff);
                break;

            case 32:
                *pslp++ = (BYTE)((c & 0xff0000)>>16);
                *pslp++ = (BYTE)((c & 0xff00)>>8);
                *pslp++ = (BYTE)(c & 0xff);
                pslp++;
                break;
            }

        } else {
            BYTE *ppx = pb+3*((bx+m_loff) % (2*map_defs[m_curmap].xdim));

            MYTRACE(_T("l this=%p m_buf=%p x0=%d, loff=%d ppx=%p\n"), this, m_buf, x0, m_loff, ppx);

            switch (pddsd->ddpfPixelFormat.dwRGBBitCount) {
            case 8:
                *pslp++ = 0;
                break;

            case 24:
                *pslp++ = ppx[2];
                *pslp++ = ppx[1];
                *pslp++ = ppx[0];
                break;

            case 32:
                *pslp++ = ppx[2];
                *pslp++ = ppx[1];
                *pslp++ = ppx[0];
                pslp++;
                break;
            }

        }

    }

#if 0
    long dy = (2*y-yb-yt)/2;
    long rxy = (long)sqrt((double)(m_rx*m_rx)*(1.0-((double)(dy*dy)/(double)(m_ry*m_ry))));

    pslp = psl + (x0-rxy)*pixsz;
    BYTE *pslp2 = psl + (x0+rxy)*pixsz;

    switch (pddsd->ddpfPixelFormat.dwRGBBitCount) {
    case 8:
        *pslp++ = 13;
        *pslp2++ = 13;
        break;

    case 24:
        *pslp++ = 255;
        *pslp++ = 255;
        *pslp++ = 255;
        *pslp2++ = 255;
        *pslp2++ = 255;
        *pslp2++ = 255;
        break;

    case 32:
        *pslp++ = 255;
        *pslp++ = 255;
        *pslp++ = 255;
        pslp++;
        *pslp2++ = 255;
        *pslp2++ = 255;
        *pslp2++ = 255;
        pslp2++;
        break;
    }
#endif
}

void CMainFrame::HLine(HDC hdc, int yt, int yb, int y, int xl, int xr)
{
    int by = (map_defs[m_curmap].ydim-1)*(y-yt)/(yb-yt);

    BYTE *p = m_buf[m_curmap]+(map_defs[m_curmap].xdim*2*map_defs[m_curmap].bpp*by);

    for ( int x=xl ; x<=xr ; x++ ) {
        int bx = (map_defs[m_curmap].xdim-1)*(x-xl)/(xr-xl);
        SetPixelV(hdc, x, y, mapcols[p[(bx+m_loff) % (2*map_defs[m_curmap].xdim)]]);
    }
}

//#define DOUBLE_BUFFER
#define DIRECTDRAW_PAINT

void CMainFrame::DDPaint(HDC hdc, RECT &rc)
{
    if (!m_bDDStarted) {
        InitDD();
        m_bDDStarted = true;
    }

    DDSURFACEDESC2 ddsdc;
    memset(&ddsdc, 0, sizeof(ddsdc));
    ddsdc.dwSize = sizeof(ddsdc);
    HRESULT hr = m_spDD->GetDisplayMode(&ddsdc);

    RECT src = rc;
    ClientToScreen(&src);

    hr = m_spDD->TestCooperativeLevel();
    switch (hr) {
    case DDERR_EXCLUSIVEMODEALREADYSET:
        return;

    case DDERR_WRONGMODE:
        InitDD();
        return;
    }

    DDBLTFX ddbltfx;
    memset(&ddbltfx, 0, sizeof(ddbltfx));
    ddbltfx.dwSize = sizeof(ddbltfx);
    ddbltfx.dwFillColor = RGB(0, 0, 0);
    hr = m_spS2->Blt(NULL, NULL, NULL, DDBLT_COLORFILL, &ddbltfx);

    //

    DDSURFACEDESC2 ddsdp;
    memset(&ddsdp, 0, sizeof(ddsdp));
    ddsdp.dwSize = sizeof(ddsdp);
    hr = m_spS2->Lock(NULL, &ddsdp, DDLOCK_WAIT, NULL);

    //

//    int rx = max(16, min(rc.right, rc.bottom)-16)/2;
//    int ry = rx;
//    int rx = max(0, rc.right/2 - 16);
//    int ry = max(0, rc.bottom/2 - 16);

    SIZE scrszmm = { GetDeviceCaps(hdc, HORZSIZE), GetDeviceCaps(hdc, VERTSIZE) };
    SIZE scrszpx = { GetDeviceCaps(hdc, HORZRES), GetDeviceCaps(hdc, VERTRES) };

    int rx = max(0, rc.right/2 - 16);
    int ry = rx * scrszmm.cx * scrszpx.cy / (scrszpx.cx * scrszmm.cy);

    if (ry>(rc.bottom/2 - 16)) {
        ry = max(0, rc.bottom/2 - 16);
        rx = ry * scrszmm.cy * scrszpx.cx / (scrszpx.cy * scrszmm.cx);
    }

    int x0 = rc.right/2;
    int y0 = rc.bottom/2;

    if (rx && ry) {

        int xr2s = 2*rx*rx;
        int yr2s = 2*ry*ry;

        int x = rx;
        int y = 0;
        int xi = ry*ry*(1-2*rx);
        int yi = rx*rx;
        int e = 0;
        int sx = yr2s*rx;
        int sy = 0;

        while (sx>=sy) {
            DDHLine(&ddsdp, y0-ry, y0+ry, y0+y, x0, x);
            if (y) DDHLine(&ddsdp, y0-ry, y0+ry, y0-y, x0, x);

            y++;
            sy += xr2s;
            e += yi;
            yi += xr2s;

            if (2*e+xi>0) {
                x--;
                sx -= yr2s;
                e += xi;
                xi += yr2s;
            }
        }

        x = 0;
        y = ry;
        xi = ry*ry;
        yi = rx*rx*(1-2*ry);
        e = 0;
        sx = 0;
        sy = xr2s*ry;

        while (sx<=sy) {
            x++;
            sx += yr2s;
            e += xi;
            xi += yr2s;

            if (2*e+yi>0) {
                DDHLine(&ddsdp, y0-ry, y0+ry, y0+y, x0, x-1);
                DDHLine(&ddsdp, y0-ry, y0+ry, y0-y, x0, x-1);

                y--;
                sy -= xr2s;
                e += yi;
                yi += xr2s;
            }
        }
    }

    m_x0 = x0;
    m_y0 = y0;
    m_rx = rx;
    m_ry = ry;

    //

    hr = m_spS2->Unlock(NULL);

    //

    m_spDD->WaitForVerticalBlank(DDWAITVB_BLOCKBEGIN, NULL);
    hr = m_spS1->Blt(&src, m_spS2, NULL, DDBLT_WAIT, NULL);
    if (hr==DDERR_SURFACELOST) {
        m_spS1->Restore();
        m_spS2->Restore();
    }
}

LRESULT CMainFrame::OnPaint(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
//	CPaintDC dc(m_hWnd);
    PAINTSTRUCT ps;
    BeginPaint(&ps);

    RECT rc;
    GetClientRect(&rc);

//    ATLTRACE(_T("Client Rect = %dx%d\n"), rc.right, rc.bottom);

#ifdef DIRECTDRAW_PAINT
    DDPaint(ps.hdc, rc);
#else

#ifdef DOUBLE_BUFFER
    HDC hdcM = CreateCompatibleDC(ps.hdc);
    HBITMAP hbm = CreateCompatibleBitmap(ps.hdc, rc.right, rc.bottom);
    HBITMAP hbmOld = (HBITMAP)SelectObject(hdcM, hbm);

    HDC hdc = hdcM;
#else
    HDC hdc = ps.hdc;
#endif

    FillRect(hdc, &rc, (HBRUSH)GetStockObject(BLACK_BRUSH));

    int r = max(16, min(rc.right, rc.bottom)-16)/2;
//    int r = 200;
    int x0 = rc.right/2;
    int y0 = rc.bottom/2;

    int dE = 3;
    int dSE = 5-2*r;
    int d = 1-r;
    int x = 0;
    int y = r;

//    SetPixelV(hdc, x0+x, y0-y, RGB(255, 255, 255));
//    SetPixelV(hdc, x0+x, y0+y, RGB(255, 255, 255));
//    SetPixelV(hdc, x0+y, y0-x, RGB(255, 255, 255));
//    SetPixelV(hdc, x0+y, y0+x, RGB(255, 255, 255));

    HLine(hdc, y0-r, y0+r, y0, x0-r, x0+r);

    while (y>x) {
        if (d<0) {
            d += dE;
            dE += 2;
            dSE += 2;
        } else {
            d += dSE;
            dE += 2;
            dSE += 4;

            HLine(hdc, y0-r, y0+r, y0-y, x0-x, x0+x);
            HLine(hdc, y0-r, y0+r, y0+y, x0-x, x0+x);

            y--;
        }
        x++;

        HLine(hdc, y0-r, y0+r, y0-x, x0-y, x0+y);
        HLine(hdc, y0-r, y0+r, y0+x, x0-y, x0+y);

//        SetPixelV(hdc, x0+x, y0-y, RGB(255, 255, 255));
//        SetPixelV(hdc, x0-x, y0-y, RGB(255, 255, 255));
//        SetPixelV(hdc, x0+x, y0+y, RGB(255, 255, 255));
//        SetPixelV(hdc, x0-x, y0+y, RGB(255, 255, 255));
//        SetPixelV(hdc, x0+y, y0-x, RGB(255, 255, 255));
//        SetPixelV(hdc, x0-y, y0-x, RGB(255, 255, 255));
//        SetPixelV(hdc, x0+y, y0+x, RGB(255, 255, 255));
//        SetPixelV(hdc, x0-y, y0+x, RGB(255, 255, 255));
    }

#ifdef DOUBLE_BUFFER
    BitBlt(ps.hdc, 0, 0, rc.right, rc.bottom, hdcM, 0, 0, SRCCOPY);
    SelectObject(hdcM, hbmOld);
    DeleteObject(hbm);
    DeleteDC(hdcM);
#endif

#endif

    EndPaint(&ps);


	return 0;
}