// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINFRM_H__A5124A5A_EF89_4646_A5E9_585328ED4B38__INCLUDED_)
#define AFX_MAINFRM_H__A5124A5A_EF89_4646_A5E9_585328ED4B38__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "resource.h"

#include "aboutdlg.h"

#include <ddraw.h>

_COM_SMARTPTR_TYPEDEF(IDirectDraw7, IID_IDirectDraw7);
_COM_SMARTPTR_TYPEDEF(IDirectDrawClipper, IID_IDirectDrawClipper);
_COM_SMARTPTR_TYPEDEF(IDirectDrawSurface7, IID_IDirectDrawSurface7);
_COM_SMARTPTR_TYPEDEF(IDirectDrawPalette, IID_IDirectDrawPalette);

struct map_def {
    LPCTSTR westpath;
    LPCTSTR eastpath;
    long xdim;
    long ydim;
    long bpp; // bytes per pixel
};

extern struct map_def map_defs[];

class CMainFrame : public CFrameWindowImpl<CMainFrame>, public CUpdateUI<CMainFrame>,
		public CMessageFilter, public CIdleHandler
{
public:
//	DECLARE_FRAME_WND_CLASS(NULL, IDR_MAINFRAME)

    static CFrameWndClassInfo& GetWndClassInfo()
    {
	    static CFrameWndClassInfo wc =
	    {
		    { sizeof(WNDCLASSEX), CS_VREDRAW|CS_HREDRAW, StartWindowProc,
		      0, 0, NULL, NULL, NULL, NULL, NULL, _T("MapShow2Class"), NULL },
		    NULL, NULL, IDC_ARROW, TRUE, 0, _T(""), IDR_MAINFRAME
	    };
	    return wc;
    }

    CMainFrame();
    ~CMainFrame();

    int m_nmaps;
    BYTE **m_buf;
    int **m_alpha;
    int m_curmap;

    long m_rx;
    long m_ry;
    long m_x0;
    long m_y0;

    long m_loff;
    bool m_bTimerOn;

    bool m_bInUser;
    bool m_bInDrag;
    long m_ooff;
    long m_doff;

    bool m_bDDStarted;

    IDirectDraw7Ptr m_spDD;
    IDirectDrawPalettePtr m_spP;
    IDirectDrawClipperPtr m_spC;
    IDirectDrawSurface7Ptr m_spS1;
    IDirectDrawSurface7Ptr m_spS2;

    void InitDD();

    void DDHLine(DDSURFACEDESC2 *pddsd, int yt, int yb, int y, int x0, int r);
    void HLine(HDC hdc, int yt, int yb, int y, int xl, int xr);

    void DDPaint(HDC hdc, RECT &rc);

	virtual BOOL PreTranslateMessage(MSG* pMsg)
	{
		return CFrameWindowImpl<CMainFrame>::PreTranslateMessage(pMsg);
	}

	virtual BOOL OnIdle()
	{
		return FALSE;
	}

	BEGIN_MSG_MAP(CMainFrame)
		MESSAGE_HANDLER(WM_PAINT, OnPaint)
        MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBkgnd)
        MESSAGE_HANDLER(WM_TIMER, OnTimer)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
        MESSAGE_HANDLER(WM_SIZE, OnSize)
        MESSAGE_HANDLER(WM_MOUSEWHEEL, OnMouseWheel)
        MESSAGE_HANDLER(WM_MBUTTONUP, OnMiddleClick)
        MESSAGE_HANDLER(WM_KEYDOWN, OnKeyDown)
        MESSAGE_HANDLER(WM_LBUTTONDOWN, OnLButtonDown)
        MESSAGE_HANDLER(WM_LBUTTONUP, OnLButtonUp)
        MESSAGE_HANDLER(WM_MOUSEMOVE, OnMouseMove)
		COMMAND_ID_HANDLER(ID_APP_EXIT, OnFileExit)
		COMMAND_ID_HANDLER(ID_NEXTMAP, OnNextMap)
        COMMAND_ID_HANDLER(ID_STOPSTART, OnStopStart)
		COMMAND_ID_HANDLER(ID_VIEW_STATUS_BAR, OnViewStatusBar)
		COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
		CHAIN_MSG_MAP(CUpdateUI<CMainFrame>)
		CHAIN_MSG_MAP(CFrameWindowImpl<CMainFrame>)
	END_MSG_MAP()

	BEGIN_UPDATE_UI_MAP(CMainFrame)
		UPDATE_ELEMENT(ID_VIEW_STATUS_BAR, UPDUI_MENUPOPUP)
	END_UPDATE_UI_MAP()

	LRESULT OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
    {
        if (m_bDDStarted) InitDD();

        bHandled = FALSE;
        return 0;
    }

	LRESULT OnTimer(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnMouseWheel(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnMiddleClick(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnKeyDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnLButtonDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnLButtonUp(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnMouseMove(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);

	LRESULT OnEraseBkgnd(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);

    LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnFileExit(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		PostMessage(WM_CLOSE);
		return 0;
	}

	LRESULT OnNextMap(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
    {
        if (m_curmap==-1) return 0;

        int nextmap = (m_curmap+1) % m_nmaps;
        while (nextmap!=m_curmap && !m_buf[nextmap]) nextmap = (nextmap+1) % m_nmaps;

        m_loff = m_loff*map_defs[nextmap].xdim/map_defs[m_curmap].xdim;
        m_curmap = nextmap;

        if (!m_bTimerOn) Invalidate();

        return 0;
    }

	LRESULT OnStopStart(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
    {
        m_bTimerOn = !m_bTimerOn;

        return 0;
    }

	LRESULT OnViewStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		BOOL bVisible = !::IsWindowVisible(m_hWndStatusBar);
		::ShowWindow(m_hWndStatusBar, bVisible ? SW_SHOWNOACTIVATE : SW_HIDE);
		UISetCheck(ID_VIEW_STATUS_BAR, bVisible);
		UpdateLayout();
		return 0;
	}

	LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CAboutDlg dlg;
		dlg.DoModal();
		return 0;
	}
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINFRM_H__A5124A5A_EF89_4646_A5E9_585328ED4B38__INCLUDED_)
